<?php

namespace Drupal\sessionless\KeyStorage;

use Drupal\Core\State\StateInterface;
use Drupal\sessionless\WebTokenAlgorithms\WebTokenAlgorithmsInterface;
use Jose\Component\Core\JWK;

/**
 * Key Storage
 *
 * @internal
 */
final class KeyStorage implements KeyStorageInterface {

  public function __construct(
    protected WebTokenAlgorithmsInterface $tokenAlgorithms,
    protected StateInterface $state,
  ) {}

  public function getSignatureKey(): JWK {
    $stateKey = "sessionless.key.signature";
    $key = $this->state->get($stateKey);
    if (!$key) {
      $key = $this->tokenAlgorithms->getSignatureKey();
      $this->state->set($stateKey, $key);
    }
    return $key;
  }

  public function getEncryptionKey(): JWK {
    $stateKey = "sessionless.key.encryption";
    $key = $this->state->get($stateKey);
    if (!$key) {
      $key = $this->tokenAlgorithms->getEncryptionKey();
      $this->state->set($stateKey, $key);
    }
    return $key;
  }

}
