<?php

declare(strict_types=1);
namespace Drupal\Tests\sessionless\Unit;

use Drupal\Core\KeyValueStore\KeyValueMemoryFactory;
use Drupal\Core\State\State;
use Drupal\sessionless\KeyStorage\KeyStorage;
use Drupal\sessionless\WebTokenAlgorithms\WebTokenAlgorithms;
use Drupal\sessionless\WebTokenService\WebTokenEncryptAndSignService;
use Drupal\sessionless\WebTokenService\WebTokenOnlySignService;
use Drupal\sessionless\WebTokenService\WebTokenServiceInterface;
use Drupal\Tests\sessionless\Traits\ClassWithPrivateProperty;
use Drupal\Tests\UnitTestCase;
use Drupal\TestTools\Random;

/**
 * @group sessionless
 */
final class WebTokenServiceUnitTest extends UnitTestCase {

  protected function setUp(): void {
    parent::setUp();
  }

  /**
   * @dataProvider provideCryptoService
   */
  public function testCryptoService(WebTokenServiceInterface $tokenService): void {
    // Class with private property contains nul bytes when serialized.
    $data = [
      'randomObject' => Random::object(10),
      'classWithPrivateString' => new ClassWithPrivateProperty(Random::string(500)),
    ];
    $token = $tokenService->dump($data);
    $this->assertIsString($token);
    // $this->assertSame('', $token);
    $this->assertMatchesRegularExpression('~^[A-Za-z0-9-_=]+\.[A-Za-z0-9-_=]+\.?[A-Za-z0-9-_.+/=]*$~', $token);
    $recoveredData = $tokenService->parse($token);
    // The recreated object is equal, but not same.
    $this->assertEquals($data, $recoveredData);
    $this->assertNotSame($data, $recoveredData);
  }

  public function provideCryptoService() {
    yield 'onlySigned' => [$this->getTokenService(false)];
    yield 'encryptedSigned' => [$this->getTokenService(true)];
  }

  public function getTokenService(bool $encrypt): WebTokenServiceInterface {
    $state = new State(new KeyValueMemoryFactory());
    $cryptoWorker = new WebTokenAlgorithms();
    $cryptoKeyStorage = new KeyStorage($cryptoWorker, $state);
    if ($encrypt) {
      $tokenService = new WebTokenEncryptAndSignService($cryptoKeyStorage, $cryptoWorker);
    }
    else {
      $tokenService = new WebTokenOnlySignService($cryptoKeyStorage, $cryptoWorker);

    }
    return $tokenService;
  }

}
